@@ -1,7 +1,7 @@
 /*GPL*START*
  * mp3check - check mp3 file for consistency and print infos
  * 
- * Copyright (C) 1998-2001 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 1998-2002 by Johannes Overmann <overmann@iname.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,13 +29,8 @@
 #include "crc16.h"
 #include "id3tag.h"
 #include "tfiletools.h"
+#include "config.h"
 
-// defines:
-
-// define this if you compile mp3check for a bigendian machine (like m68k, ppc ...):
-//#define BIGENDIAN
-
-#define VERSION "0.7.3"
 
 
 #define ONLY_MP3 "mp3,MP3"
@@ -51,7 +46,7 @@ const char *options[] = {
    // options
    "name=list             , type=switch, char=l, help='list parameters by examining the first valid header and size', headline=mode:",
    "name=compact-list     , type=switch, char=c, help='list parameters of one file per line in a very compact format: "
-     "version (l=1.0, L=2.0), layer, sampling frequency [kHz] (44=44.1), bitrate [kBit/s], mode (js=joint stereo, st=stereo, sc=single channel, dc=dual channel), "
+     "version (l=1.0, L=2.0), layer, sampling frequency [kHz] (44=44.1), bitrate [kbit/s], mode (js=joint stereo, st=stereo, sc=single channel, dc=dual channel), "
      "emphasis (n=none, 5=50/15 usecs, J=CCITT J.17), COY (has [C]rc, [O]riginal, cop[Y]right), length [min:sec], filename (poss. truncated)'",
    "name=error-check      , type=switch, char=e, help='check crc and headers for consistency and print several error messages'",
    "name=max-errors       , type=int   , char=m, param=N, lower=0, help='with -e: set maximum number of errors N to print per file (default 0==infinity)'",
@@ -76,7 +71,7 @@ const char *options[] = {
    "name=ign-non-ampeg    , type=switch, char=N, help='ignore non audio mpeg streams'",
    "name=ign-truncated    , type=switch, char=T, help='ignore truncated last frames'",
    "name=ign-junk-start   , type=switch, char=S, help='ignore junk before first frame'",
-   "name=ign-bitrate-sw   , type=switch, char=B, help='ignore bitrate switching'",
+   "name=ign-bitrate-sw   , type=switch, char=B, help='ignore bitrate switching and enable VBR support'",
    "name=ign-constant-sw  , type=switch, char=W, help='ignore switching of constant parameters, such as sampling frequency'",
    "name=show-valid       , type=switch,       , help='print the message \\'valid audio mpeg stream\\' for all files which appear to be error free (after ignoring errors)",
 					 
@@ -225,7 +220,7 @@ int samp_2_tab[4]={22050, 24000, 16000, 50000};
 
 const unsigned int CONST_MASK = 0xffffffff;
 struct Header {
-#ifdef BIGENDIAN
+#ifdef WORDS_BIGENDIAN
    unsigned int
      syncword: 12,         // fix must 0xfff
      ID: 1,                // fix 1==mpeg1.0 0==mpeg2.0
@@ -311,38 +306,38 @@ struct Header {
    enum {STEREO, JOINT_STEREO, DUAL_CHANNEL, SINGLE_CHANNEL};
    const char *mode_str() const {
       switch(mode) {
-       case 0: return "stereo";
-       case 1: return "joint stereo";
-       case 2: return "dual channel";
-       case 3: return "single chann";
+       case STEREO:         return "stereo";
+       case JOINT_STEREO:   return "joint stereo";
+       case DUAL_CHANNEL:   return "dual channel";
+       case SINGLE_CHANNEL: return "single chann";
       }
       return 0;
    }
    const char *short_mode_str() const {
       switch(mode) {
-       case 0: return "st";
-       case 1: return "js";
-       case 2: return "dc";
-       case 3: return "sc";
+       case STEREO:         return "st";
+       case JOINT_STEREO:   return "js";
+       case DUAL_CHANNEL:   return "dc";
+       case SINGLE_CHANNEL: return "sc";
       }
       return 0;
    }
    enum {emp_NONE, emp_50_15_MICROSECONDS, emp_RESERVED, emp_CCITT_J_17};
    const char *emphasis_str() const {
       switch(emphasis) {
-       case 0: return "no emph";
-       case 1: return "50/15us";
-       case 2: return "reservd";
-       case 3: return "C. J.17";
+       case emp_NONE:               return "no emph";
+       case emp_50_15_MICROSECONDS: return "50/15us";
+       case emp_RESERVED:           return "reservd";
+       case emp_CCITT_J_17:         return "C. J.17";
       }
       return 0;
    }
    const char *short_emphasis_str() const {
       switch(emphasis) {
-       case 0: return "n";
-       case 1: return "5";
-       case 2: return "!";
-       case 3: return "J";
+       case emp_NONE:               return "n";
+       case emp_50_15_MICROSECONDS: return "5";
+       case emp_RESERVED:           return "!";
+       case emp_CCITT_J_17:         return "J";
       }
       return 0;
    }
@@ -367,7 +362,7 @@ struct Header {
 inline Header get_header(const unsigned char *p) {
    Header h;
    unsigned char *q = (unsigned char *)&h;
-#ifdef BIGENDIAN
+#ifdef WORDS_BIGENDIAN
       q[0]=p[0];
       q[1]=p[1];
       q[2]=p[2];
@@ -385,7 +380,7 @@ inline Header get_header(const unsigned char *p) {
 // set header to pointer
 inline void set_header(unsigned char *p, Header h) {
    unsigned char *q = (unsigned char *)&h;
-#ifdef BIGENDIAN
+#ifdef WORDS_BIGENDIAN
       p[0]=q[0];
       p[1]=q[1];
       p[2]=q[2];
@@ -399,6 +394,25 @@ inline void set_header(unsigned char *p, Header h) {
 }
 
 
+// set header from header
+// preserves padding bit and mode extension (under conditions),
+// among other things
+void set_header(Header &to, Header from) {
+   if(to.mode!=from.mode)
+   {
+      to.mode            = from.mode;
+      to.mode_extension  = from.mode_extension;
+   }
+   to.ID                 = from.ID;
+   to.layer_index        = from.layer_index;
+   to.protection_bit     = from.protection_bit;
+   to.sampling_frequency = from.sampling_frequency;
+   to.copyright          = from.copyright;
+   to.original           = from.original;
+   to.emphasis           = from.emphasis;
+}
+
+
 // set crc to pointer
 inline bool set_crc_value(unsigned char *p, unsigned short c) {
     p[1] = (unsigned char) c;
@@ -407,7 +421,7 @@ inline bool set_crc_value(unsigned char *p, unsigned short c) {
 }
 
 
-// return lenght of frame in bytes
+// return length of frame in bytes
 inline int frame_length(Header h) {
    if(h.version() == 1.0) {
       switch(h.layer()) {
@@ -427,6 +441,12 @@ inline int frame_length(Header h) {
 }
 
 					 
+// return duration of frame in ms
+inline double frame_duration(Header h) {
+   return (((double)frame_length(h)*8) / h.bitrate());
+}
+
+
 // return next pos of min_valid sequential valid and constant header 
 // or -1 if not found
 inline int find_next_header(const unsigned char *p, int len, int min_valid) {
@@ -524,7 +544,8 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
    int start = find_next_header(p, len, min_valid);
    int rest = len;
    int frame=0;
-   int l,s;
+   double time=0.0;
+   int l=0,s;
    Tagv1* tag;
    
    if(start<0) {
@@ -595,37 +616,85 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
 	    // invalid header 
 	    
 	    // search for next valid header
-	    s = find_next_header(p, rest, min_valid);
+	    if(!l) {
+	       printf("ERROR! Invalid header with no previous frame. Needs debuging.\n");
+	    }
+	    // search within previous frame
+	    p-=(l-4);
+	    start-=(l-4);
+	    rest+=(l-4);
 	    
-	    if(s<0) break; // error: junk at eof
+	    // first look for any isolated frame with the same header
+	    s = find_next_header(p, rest, 1);
+	    if(s<0) { // error: junk at eof
+	       p+=(l-4);
+	       start+=(l-4);
+	       rest-=(l-4);
+	       break;
+	    }
 	    
+	    // else look for a regular stream
+      	    h = get_header(p+s);
+      	    if(!head.sameConstant(h))
+	       s = find_next_header(p, rest, min_valid);
+	    if(s<0) { // error: junk at eof
+	       p+=(l-4);
+	       start+=(l-4);
+	       rest-=(l-4);
+	       break;
+	    }
+      	    
 	    if(!ign_sync) {
-	       fmes(name, "frame %s%5d%s: %sinvalid header at%s %s0x%08x%s (%s0x%08x%s), skipping %s%d%s byte%s\n", 
-		    cval, frame, cnor, cerror, cnor, cval, start, cnor, cval, 
-		    h.get_int(), cnor, cval, s, cnor, (s>1)?"s":"");
-	       errors++;
+	       if(s<l-4) {
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sframe too short%s at %s0x%08x%s, %s%d%s byte%s mising\n", 
+		       cval, frame - 1, cnor,
+		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		       cerror, cnor, cval, start - 4, cnor,
+		       cval, l-4-s, cnor, (l-4-s>1)?"s":"");
+		  errors++;
+	       } else {
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sframe too long%s at %s0x%08x%s, skipping %s%d%s byte%s at %s0x%08x%s\n", 
+		       cval, frame - 1, cnor,
+		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		       cerror, cnor, cval, start - 4, cnor,
+		       cval, s-l+4, cnor, (s-l+4>1)?"s":"",
+		       cval, start - 4 + l, cnor);
+		  errors++;
+	       }
 	    }	    
 
 	    // try to fix header including sync information
-	    if(fix_headers) {
+	    if(fix_headers && (s>l-4)) {
+	       unsigned int old_padding_bit = head.padding_bit;
 	       head.padding_bit = 0;
-	       if(s == frame_length(head)) {
-		  fmes(name, "frame %s%5d%s: %sfixing header (including sync)%s\n",
-		       cval, frame, cnor, cerror, cnor);
-		  set_header((unsigned char *)p, head);
+	       if(s-l+4 == frame_length(head)) {
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sfixing header (including sync)%s\n",
+		       cval, frame, cnor,
+		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		       cerror, cnor);
+		  set_header((unsigned char *)(p+l-4), head);
 		  frame++; // we just created a new frame
+		  time+=frame_duration(head);
+		  l = s-l+4;
 	       } else {
 		  head.padding_bit = 1;
-		  if(s == frame_length(head)) {
-		     fmes(name, "frame %s%5d%s: %sfixing header (including sync)%s\n",
-			  cval, frame, cnor, cerror, cnor);
-		     set_header((unsigned char *)p, head);
+		  if(s-l+4 == frame_length(head)) {
+		     fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sfixing header (including sync)%s\n",
+			  cval, frame, cnor,
+			  cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+			  cerror, cnor);
+		     set_header((unsigned char *)(p+l-4), head);
 		     frame++; // we just created a new frame
+		     time+=frame_duration(head);
+		     l = s-l+4;
+		  } else {
+		     // prevent possible side effect
+		     head.padding_bit = old_padding_bit;
 		  } 		  
 	       }
 	    } 
 
-	    // skip s invalid bytes
+	    // position on next frame
 	    p += s;
 	    rest -= s;
 	    start += s;
@@ -635,24 +704,92 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
 	    // check for constant parameters
 	    if(!head.sameConstant(h)) {
 	       if(!ign_const) {
-		  fmes(name, "frame %s%5d%s: %sconstant parameter switching%s (%s0x%08x%s!=%s0x%08x%s)\n", 
-		       cval, frame, cnor, cerror, cnor, cval, head.get_int()&CONST_MASK, 
-		       cnor, cval, h.get_int()&CONST_MASK, cnor);
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sconstant parameter switching%s at %s0x%08x%s (%s0x%08x%s -> %s0x%08x%s)\n",
+		       cval, frame, cnor,
+		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		       cerror, cnor,
+		       cval, start, cnor,
+		       cval, head.get_int()&CONST_MASK, cnor,
+		       cval, h.get_int()&CONST_MASK, cnor);
+		  if(h.ID!=head.ID)
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %sMPEG version switching%s (MPEG %s%1.1f%s -> MPEG %s%1.1f%s)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.version(), cnor,
+			    cval, h.version(), cnor);
+		  if(h.layer_index!=head.layer_index)
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %sMPEG layer switching%s (layer %s%1d%s -> layer %s%1d%s)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.layer(), cnor,
+			    cval, h.layer(), cnor);
+		  if(h.samp_rate()!=head.samp_rate())
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %ssampling frequency switching%s (%s%f%skHz -> %s%f%skHz)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.samp_rate(), cnor,
+			    cval, h.samp_rate(), cnor);
+		  if(h.mode!=head.mode)
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %smode switching%s (%s%s%s -> %s%s%s)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.mode_str(), cnor,
+			    cval, h.mode_str(), cnor);
+		  if(h.protection_bit!=head.protection_bit)
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %sprotection bit switching%s (%s%s%s -> %s%s%s)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.protection_bit?"no crc":"crc", cnor,
+			    cval, h.protection_bit?"no crc":"crc", cnor);
+		  if(h.copyright!=head.copyright)
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %scopyright bit switching%s (%s%s%s -> %s%s%s)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.copyright?"copyright":"no copyright", cnor,
+			    cval, h.copyright?"copyright":"no copyright", cnor);
+		  if(h.original!=head.original)
+		     printf("frame %s%5d%s/%s%2u:%02u%s:   %soriginal bit switching%s (%s%s%s -> %s%s%s)\n",
+		            cval, frame, cnor,
+		            cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		            cerror, cnor,
+			    cval, head.original?"original":"not original", cnor,
+			    cval, h.original?"original":"not original", cnor);
 		  errors++;
 	       }
 	       if(fix_headers) {
-		  fmes(name, "frame %s%5d%s: %sfixing header%s\n", 
-		       cval, frame, cnor, cerror, cnor);
-		  set_header((unsigned char *)p, head);
-		  h = head;
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sfixing header%s\n", 
+		       cval, frame, cnor,
+		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		       cerror, cnor);
+		  // fix only what should be
+		  set_header(h, head);
+		  set_header((unsigned char *)p, h);
 	       } 
 	    }
 	    if(head.bitrate_index != h.bitrate_index) {
 	       if(!ign_bit) {
-		  fmes(name, "frame %s%5d%s: %sbitrate switching from %s%d%s to %s%d%s\n", 
-		       cval, frame, cnor, cerror, cval, 
-		       head.bitrate(), cerror, cval, h.bitrate(), cnor);  		     
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sbitrate switching%s (%s%d%s -> %s%d%s)\n", 
+		       cval, frame, cnor,
+		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		       cerror, cnor, 
+		       cval, head.bitrate(), cnor,
+		       cval, h.bitrate(), cnor);
 		  errors++;
+		  if(fix_headers) {
+		     fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sfixing header%s\n", 
+			  cval, frame, cnor,
+			  cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+			  cerror, cnor);
+		     // fix only what should be
+		     h.bitrate_index=head.bitrate_index;
+		     set_header((unsigned char *)p, h);
+		  } 
 	       }
 	    }
 	    head = h;
@@ -703,8 +840,10 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
 		     if(fix_crc) {
 			fixed_crc = set_crc_value((unsigned char *) &(p[4]), crc.crc());
 		     }
-		     fmes(name, "frame %s%5d%s: %scrc error%s (%s0x%04x%s!=%s0x%04x%s)%s\n",
-			  cval, frame, cnor, cerror, cnor, 
+		     fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %scrc error%s (%s0x%04x%s!=%s0x%04x%s)%s\n",
+			  cval, frame, cnor,
+			  cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		          cerror, cnor, 
 			  cval, c, cnor, cval, crc.crc(), cnor, fixed_crc ? " fixed" : "");
 		     errors++;
 		  }
@@ -718,13 +857,16 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
 	    rest -= l;
 	    start += l;
 	    frame++;
+	    time+=frame_duration(h);
 	 }
 	 
 	 // maximum number of error reached?
 	 if(max_errors && (errors >= max_errors))
 	 {
-	    fmes(name, "frame %s%5d%s: %smaximum number of errors exceeded%s\n", 
-		 cval, frame, cnor, cerror, cnor);
+	    fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %smaximum number of errors exceeded%s\n", 
+		 cval, frame, cnor,
+		 cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		 cerror, cnor);
 	    rest = 0;
 	    break;
 	 }
@@ -733,8 +875,10 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
       // check for truncated file
       if(rest < 0) {
 	 if(!ign_trunc) {
-	    fmes(name, "frame %s%5d%s: %sfile truncated%s, %s%d%s byte%s missing for last frame (%s0x%08x%s)\n", 
-		 cval, frame, cnor, cerror, cnor, cval,-rest, cnor, (-rest)>1?"s":"", cval, start, cnor);	    
+	    fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sfile truncated%s, %s%d%s byte%s missing for last frame\n", 
+		 cval, frame, cnor,
+		 cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		 cerror, cnor, cval, -rest, cnor, (-rest)>1?"s":"");	    
 	    errors++;
 	 }
       }
@@ -742,8 +886,10 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
       // check for trailing junk
       if(rest > 0) {
 	 if(!ign_end) {
-	    fmes(name, "frame %s%5d%s: %s%d%s %sbyte%s of junk after last frame%s at %s0x%08x%s\n", 
-		 cval, frame, cnor, cval, rest, cnor, cerror, (rest>1)?"s":"", cnor, cval, start, cnor);
+	    fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %s%d%s %sbyte%s of junk after last frame%s at %s0x%08x%s\n", 
+		 cval, frame, cnor,
+		 cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
+		 cval, rest, cnor, cerror, (rest>1)?"s":"", cnor, cval, start, cnor);
 	    errors++;
 	    // check for possible id3 tags within the junk
 	    while(rest >= 128)
@@ -808,8 +954,8 @@ bool anomaly_check(const char *name, const unsigned char *p, int len, bool err_c
 	 }
       }
       if(!ano_any_bit) {
-	 if((h.bitrate()!=112) && (h.bitrate()!=128)) {
-	    fmes(name, "%sanomaly%s: bitrate %s%3dkBit/s%s\n", 
+	 if(h.bitrate()!=128) {
+	    fmes(name, "%sanomaly%s: bitrate %s%3dkbit/s%s\n", 
 		 cano, cnor, cval, h.bitrate(), cnor); 
 	    had_ano = true;
 	 }
@@ -845,6 +991,46 @@ bool anomaly_check(const char *name, const unsigned char *p, int len, bool err_c
 }
 					
 
+// returns the stream duration in ms
+// also returns minimum, maximum and average bitrates if told to
+unsigned int stream_duration(const unsigned char *p, int len, unsigned short int *minbr, unsigned short int *maxbr, unsigned short int *avgbr) {
+   int next = find_next_header(p, len, min_valid);
+   int rest = len - next;
+   double duration = 0.0;
+   unsigned short int min = 2048, max = 0;
+   unsigned long int bytes = 0;
+
+   if(next<0) return 0;
+
+   while(rest>=4) {
+      Header h=get_header(p+next);
+      if(!h.isValid()) {
+      	 int old = next;
+	 next = find_next_header(p+old, rest, min_valid);
+	 if(next<0) break;
+	 rest -= next;
+	 next += old;
+      }
+      else
+      {
+	 int l=frame_length(h);
+	 int br=h.bitrate();
+	 if(br>max) max=br;
+	 if(br<min) min=br;
+	 bytes+=l;
+	 next+=l;
+	 rest-=l;
+	 duration+=frame_duration(h);
+      }
+   }
+
+   if(minbr!=NULL) { *minbr = min; }
+   if(maxbr!=NULL) { *maxbr = max; }
+   if(avgbr!=NULL) { *avgbr = (bytes * 8) / (unsigned int)duration; }
+   return (unsigned int)duration;
+}
+
+
 // return true if junk was found and cut
 bool cut_junk_end(const char *name, const unsigned char *p, int len, const unsigned char *free_p, int fd, int& err) {
 // this is basically the error_check routine which treats only the trainling junk case
@@ -1187,10 +1373,10 @@ int main(int argc, char *argv[]) {
    int checked=0;
    int num_ano=0;
    CRC16 crc(CRC16::CRC_16);   
-   FILE *log = 0;
+   FILE *log = NULL;
    if(!ac.getString("log-file").empty()) {
       log = fopen(ac.getString("log-file").c_str(), "a");
-      if(log==0)
+      if(log==NULL)
 	userError("can't open logfile '%s'!\n", ac.getString("log-file").c_str());
    }
    for(size_t i = 0; i < filelist.size(); i++) {
@@ -1260,7 +1446,7 @@ int main(int argc, char *argv[]) {
       if(ac("list")||ac("compact-list")||ac("raw-list")) {
 	 // speed up list of very large files (like *.wav)
 	 int maxl = 128*1024; // search max 128k
-	 int start = find_next_header(p, len<=maxl?len:maxl, min_valid);
+	 int start = find_next_header(p, len<maxl?len:maxl, min_valid);
 	 if(start<0) {
 	    if(!ign_noamp) {
 	       if(ac("raw-list")) {
@@ -1274,7 +1460,8 @@ int main(int argc, char *argv[]) {
 	    }
 	 } else {
 	    Header h = get_header(p+start);
-	    unsigned int l_min = len/(h.bitrate()/8);
+	    unsigned short int minbr = 0, maxbr = 0, avgbr;
+	    unsigned int l_min = (ign_bit?stream_duration(p, len, &minbr, &maxbr, &avgbr):len/(h.bitrate()/8));
 	    unsigned int l_mil = l_min%1000;
 	    l_min/=1000;
 	    unsigned int l_sec = l_min%60;
@@ -1289,54 +1476,69 @@ int main(int argc, char *argv[]) {
 	    if(tag->isValid())
 	    	tag_version=tag->version();
 	    if(ac("list")) {
+	       unsigned int xwidth = 0;
 	       tstring n = single_line?tstring(name):tstring(name).shortFilename(columns-1);
-	       fmes(name, "mpeg %s%3.1f%s layer %s%d%s %s%2.1f%skHz %s%3d%skBit %s%-12.12s%s %s%-7.7s%s %s%s%s %s%s%s %s%s%s%s%s:%02u.%02u%s",
+	       fmes(name, "mpeg %s%3.1f%s layer %s%d%s %s%2.1f%skHz %s%3d%skbps",
 		      h.version()==1.0?cval:cano, h.version(), cnor, 
 		      h.layer()==3?cval:cano, h.layer(), cnor, 
 		      h.samp_rate()==44.1?cval:cano, h.samp_rate(), cnor, 
-		      h.bitrate()==128?cval:cano, h.bitrate(), cnor,
+		      (h.bitrate()==128&&minbr==maxbr)?cval:cano, minbr!=maxbr?avgbr:h.bitrate(), cnor);
+	       if(ign_bit && columns>=83) {
+	          printf(" %s%s%s",
+		         minbr!=maxbr?cano:cval,minbr!=maxbr?"VBR":"CBR",cnor);
+		  xwidth+=4;
+	       } 
+	       printf(" %s%-12.12s%s %s%-7.7s%s %s%s%s %s%s%s %s%s%s %s%s:%02u.%02u%s",
 		      h.mode==Header::JOINT_STEREO?cval:cano, h.mode_str(), cnor,
 		      h.emphasis==Header::emp_NONE?cval:cano, h.emphasis_str(), cnor,
 		      h.protection_bit?cano:cval, h.protection_bit?"---":"crc", cnor, 
-		      h.original?cval:cano, h.original?"orig":"copy", cnor,
-		      cval, h.copyright?"cprgt":"-----", cnor,
+		      h.original?cval:cano, h.original?"orig":"----", cnor,
+		      cval, h.copyright?"copy":"----", cnor,
 		      cval, l_str.c_str(), l_sec, l_mil/10, cnor);
-	       if(columns>=92)
-	       {
+	       if(columns>=87+xwidth) {
 	         if(tag_version)
-		   printf(" %sid3 tag %s%1u.%1u%s", cnor, cval, tag_version>>8,
+		   printf(" id3 %s%1u.%1u%s", cval, tag_version>>8,
 		          tag_version&0xff,cnor);
+//		 xwidth+=8;
 	       }
 	       printf("\n");
 	    } else if(ac("compact-list")) {
-	       tstring n = tstring(name).shortFilename(columns-(columns>=82?28:26));
-		   n.replaceUnprintable(only_ascii);
-	       printf("%s%c%s%s%d%s %s%2.0f%s %s%3d%s %s%s%s %s%s%s %s%s%s%s%s%s%s%s%s",
+	       unsigned int xwidth = 0;
+	       printf("%s%c%s%s%d%s %s%2.0f%s %s%3d%s",
 		      h.version()==1.0?cval:cano, h.version()==1.0?'l':'L', cnor, 
 		      h.layer()==3?cval:cano, h.layer(), cnor, 
 		      h.samp_rate()==44.1?cval:cano, h.samp_rate(), cnor, 
-		      h.bitrate()==128?cval:cano, h.bitrate(), cnor,
+		      (h.bitrate()==128&&minbr==maxbr)?cval:cano, minbr!=maxbr?avgbr:h.bitrate(), cnor);
+	       if(ign_bit && columns>=80) {
+	          printf("%s%c%s",
+		         minbr==maxbr?cval:cano, minbr==maxbr?' ':'V', cnor);
+		  xwidth+=1;
+	       }
+	       printf(" %s%s%s %s%s%s %s%s%s%s%s%s%s%s%s",
 		      h.mode==Header::JOINT_STEREO?cval:cano, h.short_mode_str(), cnor,
 		      h.emphasis==Header::emp_NONE?cval:cano, h.short_emphasis_str(), cnor,
 		      h.protection_bit?cano:cval, h.protection_bit?"-":"C", cnor, 
 		      h.original?cval:cano, h.original?"O":"-", cnor,
 		      cval, h.copyright?"Y":"-", cnor);
-	       if(columns>=82)
-	       {
+	       if(columns>=81+xwidth) {
 	         if(tag_version)
 		   printf(" %s%1u%s",cval,tag_version>>8,cnor);
 		 else
 		   printf(" %s-%s",cval,cnor);
+	         xwidth+=2;
 	       }
+	       tstring n = tstring(name).shortFilename(columns-(26+xwidth));
+	       n.replaceUnprintable(only_ascii);
 	       printf(" %s%3u:%02u%s %s%s%s\n",
 		      cval, l_min, l_sec, cnor, cfil, n.c_str(), cnor);  
 	    } else if(ac("raw-list")) {	       
-	       printf("valid_stream%c%.1f%c%d%c%.1f%c%d%c%s%c%s%c%s%c%s%c%s%c%u%c%u%c%u%c%s%c%c",
+	       printf("valid_stream%c%.1f%c%d%c%.1f%c%d%c%s%c%s%c%s%c%s%c%s%c%s%c%u%c%u%c%u%c%s%c%c",
 		      rawsep,
 		      h.version(), rawsep,
 		      h.layer(), rawsep, 
 		      h.samp_rate(), rawsep,
-		      h.bitrate(), rawsep,
+		      minbr!=maxbr?avgbr:h.bitrate(), rawsep,
+		      ign_bit?(minbr!=maxbr?"VBR":"CBR"):"?", rawsep,
 		      h.mode_str(), rawsep,
 		      h.emphasis_str(), rawsep,
 		      h.protection_bit?"---":"crc", rawsep,
@@ -1515,7 +1717,7 @@ int main(int argc, char *argv[]) {
    }
    
    // end
-   if(log) fclose(log);
+   if(log!=NULL) fclose(log);
    
    return (err || num_ano)?1:0;
 }
