@@ -21,7 +21,7 @@
 
 #include <ctype.h>
 #include <limits.h>
-#include <float.h>
+#include <values.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <sys/ioctl.h>
@@ -33,7 +33,7 @@
 // config:
 
 // global email of author:
-const char *TAPPCONFIG_EMAIL = "overmann@iname.com";
+const char *TAPPCONFIG_EMAIL = "Johannes.Overmann@gmx.de";
 // gpl message
 const char *TAPPCONFIG_GPL = "this program is distributed under the terms of the GNU General Public License";
 
@@ -116,7 +116,7 @@ const char *TAPPCONFIG_GPL = "this program is distributed under the terms of the
 #if 0
 const char *options[] = {
    "#usage='Usage: %n [OPTIONS and FILES] -- [FILES]\n\n"   // no comma here!
-     "this program does very strange things\n",
+   "this program does very strange things\n",
    "#trailer='\n%n version %v *** (C) 1997 by Johannes Overmann\ncomments, bugs and suggestions welcome: %e\n%gpl'",
    "#onlycl", // only command line options
    "#stopat--", // stop option scanning after a -- parameter
@@ -486,11 +486,11 @@ static tstring Range2Str(int lower, int upper) {
 }
 
 static tstring Range2Str(double lower, double upper) {
-   if((lower!=-DBL_MAX) || (upper!=DBL_MAX)) {
+   if((lower!=-MAXDOUBLE) || (upper!=MAXDOUBLE)) {
       tstring lstr;
       tstring ustr;
-      if(lower != -DBL_MAX) lstr = tstring(lower);
-      if(upper !=  DBL_MAX) ustr = tstring(upper);
+      if(lower != -MAXDOUBLE) lstr = tstring(lower);
+      if(upper !=  MAXDOUBLE) ustr = tstring(upper);
       return "[" + lstr + ".." + ustr + "]";
    } else {
       return "";
@@ -585,12 +585,12 @@ void TAppConfigItem::validate(const char *line_context) {
 	 if(!upper.toDouble(double_upper))
 	   fatalError2("%s: illegal chars for upper bound in '%s'!\n", line_context, upper.c_str());
       }
-      else double_upper = DBL_MAX;
+      else double_upper = MAXDOUBLE;
       if(!lower.empty()) {
 	 if(!lower.toDouble(double_lower)) 
 	   fatalError2("%s: illegal chars for lower bound in '%s'!\n", line_context, lower.c_str());
       }
-      else double_lower = -DBL_MAX;
+      else double_lower = -MAXDOUBLE;
       if(tOutOfRange(double_default, double_lower, double_upper)) 
 	fatalError3("%s: default value out of range! (%g not in %s)!\n", line_context, double_default, Range2Str(double_lower, double_upper).c_str());
       break;
