@@ -84,12 +84,17 @@
 // 00:45 15 Mar searchReplace max_num parameter added
 // 22:00 18 Sep palmos fixes
 
+// 2002:
+// 22:25 08 Apr expandUnpritable: allow high ISO graphical characters (ASCII 161-255), better nul_mem and zero_mem sizes for 64 bit systems
+
+
+
 
 // global static null and zero rep members
 tstring::Rep* tstring::Rep::nul = 0;
-int tstring::Rep::nul_mem[10];
+int tstring::Rep::nul_mem[(sizeof(Rep)/sizeof(int) + 1) * 2];
 tstring::Rep* tstring::Rep::zero = 0;
-int tstring::Rep::zero_mem[10];
+int tstring::Rep::zero_mem[(sizeof(Rep)/sizeof(int) + 1) * 2];
 
 
 // non inline Rep implementations
@@ -707,12 +712,12 @@ void tstring::truncate(size_t max) {
 
 void tstring::replaceUnprintable(bool only_ascii) {
    for(size_t i = 0; i < rep->len; i++) {
-      unsigned char c = (unsigned char)(*rep)[i];
+      unsigned char& c = (unsigned char)(*rep)[i];
       if(!isprint(c)) {
 	 if(c < ' ') {
-	    (*rep)[i] = '!';
+	    c = '!';
 	 } else if(only_ascii || (c < 0xa0)) {
-	    (*rep)[i] = '?';
+	    c = '?';
 	 }
       }
    }
@@ -842,7 +847,7 @@ void tstring::expandUnprintable(char quotes) {
    
    // expand each char
    for(size_t j = 0; j < rep->len; ++j, ++p) {
-      if(isprint(*p)) { // printable --> print
+      if(isprint(*p) || (((unsigned char)*p) > 160)) { // printable --> print
 	 if((*p=='\\') || (quotes && (*p==quotes))) { // backslashify backslash and quotes
 	    *(q++) = '\\';	 
 	    l++;	    
@@ -1132,7 +1137,7 @@ size_t tstring::firstOccurence(char c) const {
 
 
 
-// non member  implementation
+// non member implementation
 
 
 tvector<tstring> split(const tstring &s, const char *sep, bool allow_quoting, bool crop_space) {
